@@ -1,17 +1,22 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+from ..extern import six
 
 import functools
 import types
 import warnings
 
-from ..units.core import Unit, UnitsException
+from ..units.core import Unit, UnitsError
 from ..units.quantity import Quantity
 from ..utils import lazyproperty
+from ..utils.exceptions import AstropyUserWarning
+from ..utils.misc import InheritDocstrings
 
 __all__ = ['Constant', 'EMConstant']
 
 
-class ConstantMeta(type):
+class ConstantMeta(InheritDocstrings):
     """Metaclass for the :class:`Constant`. The primary purpose of this is to
     wrap the double-underscore methods of :class:`Quantity` which is the
     superclass of :class:`Constant`.
@@ -34,16 +39,16 @@ class ConstantMeta(type):
                 name_lower = self.name.lower()
                 instances = Constant._registry[name_lower]
                 if not self._checked_units:
-                    for inst in instances.values():
+                    for inst in six.itervalues(instances):
                         try:
                             self.unit.to(inst.unit)
-                        except UnitsException:
+                        except UnitsError:
                             Constant._has_incompatible_units.add(name_lower)
                     self._checked_units = True
 
                 if (not self.system and
                         name_lower in Constant._has_incompatible_units):
-                    systems = sorted(filter(None, instances))
+                    systems = sorted([x for x in instances if x])
                     raise TypeError(
                         'Constant {0!r} does not have physically compatible '
                         'units across all systems of units and cannot be '
@@ -57,8 +62,11 @@ class ConstantMeta(type):
 
         # The wrapper applies to so many of the __ methods that it's easier to
         # just exclude the ones it doesn't apply to
-        exclude = set(['__init__', '__str__', '__repr__'])
-        for attr, value in vars(Quantity).items():
+        exclude = set(['__new__', '__array_finalize__', '__array_wrap__',
+                       '__dir__', '__getattr__', '__init__', '__str__',
+                       '__repr__', '__hash__', '__iter__', '__getitem__',
+                       '__len__', '__nonzero__', '__quantity_subclass__'])
+        for attr, value in list(six.iteritems(vars(Quantity))):
             if (isinstance(value, types.FunctionType) and
                     attr.startswith('__') and attr.endswith('__') and
                     attr not in exclude):
@@ -67,15 +75,13 @@ class ConstantMeta(type):
         return super(ConstantMeta, mcls).__new__(mcls, name, bases, d)
 
 
+@six.add_metaclass(ConstantMeta)
 class Constant(Quantity):
     """A physical or astronomical constant.
 
     These objects are quantities that are meant to represent physical
     constants.
     """
-
-    __metaclass__ = ConstantMeta
-
     _registry = {}
     _has_incompatible_units = set()
 
@@ -85,11 +91,11 @@ class Constant(Quantity):
         instances = Constant._registry.setdefault(name_lower, {})
         if system in instances:
             warnings.warn('Constant {0!r} is already has a definition in the '
-                          '{1!r} system'.format(name, system))
+                          '{1!r} system'.format(name, system), AstropyUserWarning)
 
-        inst = super(Constant, cls).__new__(cls)
+        inst = super(Constant, cls).__new__(cls, value)
 
-        for c in instances.values():
+        for c in six.itervalues(instances):
             if system is not None and not hasattr(c.__class__, system):
                 setattr(c, system, inst)
             if c.system is not None and not hasattr(inst.__class__, c.system):
@@ -126,6 +132,18 @@ class Constant(Quantity):
                                            self.uncertainty, self.unit,
                                            self.reference))
 
+    def __quantity_subclass__(self, unit):
+        return super(Constant, self).__quantity_subclass__(unit)[0], False
+
+    def copy(self):
+        """
+        Return a copy of this `Constant` instance.  Since they are by
+        definition immutable, this merely returns another reference to
+        ``self``.
+        """
+        return self
+    __deepcopy__ = __copy__ = copy
+
     @property
     def abbrev(self):
         """A typical ASCII text abbreviation of the constant, also generally
@@ -191,8 +209,8 @@ class EMConstant(Constant):
 
     @property
     def cgs(self):
-        """Overridden for EMConstant to raise a `TypeError` emphasizing that
-        there are multiple EM extensions to CGS.
+        """Overridden for EMConstant to raise a `~.exceptions.TypeError`
+        emphasizing that there are multiple EM extensions to CGS.
         """
 
         raise TypeError("Cannot convert EM constants to cgs because there "
