@@ -27,40 +27,44 @@ class Distance(u.Quantity):
 
     This can be initialized in one of four ways:
 
-    * A distance `value` (array or float) and a `unit`
-    * A `~astropy.units.quantity.Quantity` object
+    * A distance ``value`` (array or float) and a ``unit``
+    * A `~astropy.units.Quantity` object
     * A redshift and (optionally) a cosmology.
     * Providing a distance modulus
 
     Parameters
     ----------
-    value : scalar or `~astropy.units.quantity.Quantity`
-        The value of this distance
-    unit : `~astropy.units.core.UnitBase`
-        The units for this distance, *if* `value` is not a `Quantity`.
-        Must have dimensions of distance.
+    value : scalar or `~astropy.units.Quantity`.
+        The value of this distance.
+    unit : `~astropy.units.UnitBase`
+        The units for this distance, *if* ``value`` is not a
+        `~astropy.units.Quantity`. Must have dimensions of distance.
     z : float
         A redshift for this distance.  It will be converted to a distance
         by computing the luminosity distance for this redshift given the
-        cosmology specified by `cosmology`. Must be given as a keyword argument.
-    cosmology : `~astropy.cosmology.Cosmology` or None
-        A cosmology that will be used to compute the distance from `z`.
-        If None, the current cosmology will be used (see
+        cosmology specified by ``cosmology``. Must be given as a keyword
+        argument.
+    cosmology : ``Cosmology`` or `None`
+        A cosmology that will be used to compute the distance from ``z``.
+        If `None`, the current cosmology will be used (see
         `astropy.cosmology` for details).
     distmod : float or `~astropy.units.Quantity`
         The distance modulus for this distance.
-    dtype : ~numpy.dtype, optional
+    dtype : `~numpy.dtype`, optional
         See `~astropy.units.Quantity`. Must be given as a keyword argument.
     copy : bool, optional
         See `~astropy.units.Quantity`. Must be given as a keyword argument.
 
     Raises
     ------
-    astropy.units.core.UnitsError
-        If the `unit` is not a distance.
+    `~astropy.units.UnitsError`
+        If the ``unit`` is not a distance.
     ValueError
-        If `z` is provided with a `unit` or `cosmology` is provided when `z` is
-        *not* given, or `value` is given as well as `z`
+        If value specified is less than 0 and ``allow_negative=False``.
+
+        If ``z`` is provided with a ``unit`` or ``cosmology`` is provided
+        when ``z`` is *not* given, or ``value`` is given as well as ``z``.
+
 
     Examples
     --------
@@ -74,13 +78,14 @@ class Distance(u.Quantity):
     >>> d4 = Distance(z=0.23)
     >>> d5 = Distance(z=0.23, cosmology=WMAP5)
     >>> d6 = Distance(distmod=24.47)
+    >>> d7 = Distance(Distance(10 * u.Mpc))
     """
 
     _include_easy_conversion_members = True
 
     def __new__(cls, value=None, unit=None, z=None, cosmology=None,
-                distmod=None, dtype=None, copy=True):
-        from ..cosmology import get_current
+                distmod=None, dtype=None, copy=True, allow_negative=False):
+        from ..cosmology import default_cosmology
 
         if isinstance(value, u.Quantity):
             # This includes Distances as well
@@ -96,11 +101,11 @@ class Distance(u.Quantity):
         elif value is None:
             if z is not None:
                 if distmod is not None:
-                    raise ValueError('both `z` and `distmod` given in Distance '
+                    raise ValueError('Both `z` and `distmod` given in Distance '
                                      'constructor')
 
                 if cosmology is None:
-                    cosmology = get_current()
+                    cosmology = default_cosmology.get()
 
                 ld = cosmology.luminosity_distance(z)
 
@@ -124,7 +129,7 @@ class Distance(u.Quantity):
                 else:
                     value = u.Quantity(value, u.parsec).to(unit).value
             else:
-                raise ValueError('none of `value`, `z`, or `distmod` were given'
+                raise ValueError('None of `value`, `z`, or `distmod` were given'
                                  ' to Distance constructor')
 
                 value = ld.value
@@ -137,7 +142,7 @@ class Distance(u.Quantity):
                              'in Distance constructor')
         elif unit is None:
             raise u.UnitsError('No unit was provided for Distance')
-        #"else" the baseline `value` + `unit` case
+        #"else" the baseline ``value`` + ``unit`` case
 
         unit = _convert_to_and_validate_length_unit(unit)
 
@@ -149,17 +154,18 @@ class Distance(u.Quantity):
         if value.dtype.kind not in 'iuf':
             raise TypeError("Unsupported dtype '{0}'".format(value.dtype))
 
+        if np.any(value < 0) and not allow_negative:
+            raise ValueError("Distance must be >= 0. Set the kwarg "
+                            "'allow_negative=True' to allow negative values.")
+
         return super(Distance, cls).__new__(cls, value, unit, dtype=dtype,
                                             copy=copy)
 
-    def __quantity_view__(self, obj, unit):
-        unit = _convert_to_and_validate_length_unit(unit)
-        return super(Distance, self).__quantity_view__(obj, unit)
-
-    def __quantity_instance__(self, val, unit, **kwargs):
-        unit = _convert_to_and_validate_length_unit(unit)
-        return super(Distance, self).__quantity_instance__(val, unit, **kwargs)
-
+    def __quantity_subclass__(self, unit):
+        if unit.is_equivalent(u.m):
+            return Distance, True
+        else:
+            return super(Distance, self).__quantity_subclass__(unit)[0], False
 
     @property
     def z(self):
@@ -173,14 +179,14 @@ class Distance(u.Quantity):
 
         Parameters
         ----------
-        cosmology : `~astropy.cosmology.cosmology` or None
-            The cosmology to assume for this calculation, or None to use the
-            current cosmology.
+        cosmology : ``Cosmology`` or `None`
+            The cosmology to assume for this calculation, or `None` to use the
+            current cosmology (see `astropy.cosmology` for details).
 
         Returns
         -------
         z : float
-            The redshift of this distance given the provided `cosmology`.
+            The redshift of this distance given the provided ``cosmology``.
         """
         from ..cosmology import luminosity_distance
         from scipy import optimize
@@ -192,7 +198,7 @@ class Distance(u.Quantity):
 
     @property
     def distmod(self):
-        """  The distance modulus of this distance as a Quantity """
+        """  The distance modulus of this distance as a `~astropy.units.Quantity` """
         val = 5. * np.log10(self.to(u.pc).value) - 5.
         return u.Quantity(val, u.mag)
 
@@ -214,22 +220,23 @@ class CartesianPoints(u.Quantity):
         The second cartesian coordinate.
     z : `~astropy.units.Quantity` or array-like, optional
         The third cartesian coordinate.
-    unit : `~astropy.units.UnitBase` object or None
-        The physical unit of the coordinate values. If `x`, `y`, or `z`
+    unit : `~astropy.units.UnitBase` object or `None`
+        The physical unit of the coordinate values. If ``x``, ``y``, or ``z``
         are quantities, they will be converted to this unit.
-    dtype : ~numpy.dtype, optional
+    dtype : `~numpy.dtype`, optional
         See `~astropy.units.Quantity`. Must be given as a keyword argument.
     copy : bool, optional
         See `~astropy.units.Quantity`. Must be given as a keyword argument.
 
     Raises
     ------
-    astropy.units.UnitsError
-        If the units on `x`, `y`, and `z` do not match or an invalid unit is given
+    UnitsError
+        If the units on ``x``, ``y``, and ``z`` do not match or an invalid
+        unit is given.
     ValueError
-        If `y` and `z` don't match `x`'s shape or `x` is not length-3
+        If ``y`` and ``z`` don't match ``x``'s shape or ``x`` is not length-3
     TypeError
-        If incompatible array types are passed into `x`, `y`, or `z`
+        If incompatible array types are passed into ``x``, ``y``, or ``z``
 
     """
 
@@ -240,7 +247,7 @@ class CartesianPoints(u.Quantity):
     def __new__(cls, x, y=None, z=None, unit=None, dtype=None, copy=True):
         if y is None and z is None:
             if len(x) != 3:
-                raise ValueError('input to CartesianPoints is not length 3')
+                raise ValueError('Input to CartesianPoints is not length 3')
 
             qarr = x
             if unit is None and hasattr(qarr, 'unit'):
@@ -252,7 +259,7 @@ class CartesianPoints(u.Quantity):
                     if hasattr(coo, 'unit'):
                         if unit is not None and coo.unit != unit:
                             raise u.UnitsError('Units for `x`, `y`, and `z` do '
-                                               'not match in CartesianPoints')
+                                               'not match in CartesianPoints   ')
                         unit = coo.unit
                 #if `unit`  is still None at this point, it means none were
                 #Quantties, which is fine, because it means the user wanted
@@ -268,11 +275,11 @@ class CartesianPoints(u.Quantity):
 
             qarr = [np.asarray(coo) for coo in (x, y, z)]
             if not (qarr[0].shape == qarr[1].shape == qarr[2].shape):
-                raise ValueError("shapes for x,y, and z don't match in "
+                raise ValueError("Shapes for `x`, `y`, and `z` don't match in "
                                  "CartesianPoints")
                 #let the unit be whatever it is
         else:
-            raise TypeError('Must give all of x,y, and z or just array in '
+            raise TypeError('Must give all of `x`, `y`, and `z` or just array in '
                             'CartesianPoints')
         try:
             unit = _convert_to_and_validate_length_unit(unit, True)
@@ -290,44 +297,44 @@ class CartesianPoints(u.Quantity):
         return super(CartesianPoints, cls).__new__(cls, qarr, unit, dtype=dtype,
                                             copy=copy)
 
-    def __quantity_view__(self, obj, unit):
-        unit = _convert_to_and_validate_length_unit(unit, True)
-        return super(CartesianPoints, self).__quantity_view__(obj, unit)
-
-    def __quantity_instance__(self, val, unit, **kwargs):
-        unit = _convert_to_and_validate_length_unit(unit, True)
-        return super(CartesianPoints, self).__quantity_instance__(val, unit, **kwargs)
+    def __quantity_subclass__(self, unit):
+        if unit.is_equivalent(u.m):
+            return CartesianPoints, True
+        else:
+            return super(CartesianPoints,
+                         self).__quantity_subclass__(unit)[0], False
 
     def __array_wrap__(self, obj, context=None):
         #always convert to CartesianPoints because all operations that would
         #screw up the units are killed by _convert_to_and_validate_length_unit
         obj = super(CartesianPoints, self).__array_wrap__(obj, context=context)
 
-        #always prefer self's unit
-        obj = obj.to(self.unit)
-
-        return CartesianPoints(obj.value, unit=obj.unit, copy=False)
+        #always prefer self's unit, if possible
+        if obj.unit.is_equivalent(self.unit):
+            return obj.to(self.unit)
+        else:
+            return obj
 
     @property
     def x(self):
         """
         The second cartesian coordinate as a `~astropy.units.Quantity`.
         """
-        return self[0]
+        return self.view(u.Quantity)[0]
 
     @property
     def y(self):
         """
         The second cartesian coordinate as a `~astropy.units.Quantity`.
         """
-        return self[1]
+        return self.view(u.Quantity)[1]
 
     @property
     def z(self):
         """
         The third cartesian coordinate as a `~astropy.units.Quantity`.
         """
-        return self[2]
+        return self.view(u.Quantity)[2]
 
     def to_spherical(self):
         """
@@ -335,11 +342,11 @@ class CartesianPoints(u.Quantity):
 
         Returns
         -------
-        r : astropy.units.Quantity
-            The radial coordinate (in the same units as this `CartesianPoint`).
-        lat : astropy.units.Quantity
+        r : `~astropy.units.Quantity`
+            The radial coordinate (in the same units as this `CartesianPoints`).
+        lat : `~astropy.units.Quantity`
             The spherical coordinates latitude.
-        lon : astropy.units.Quantity
+        lon : `~astropy.units.Quantity`
             The spherical coordinates longitude.
 
         """
@@ -356,13 +363,15 @@ class CartesianPoints(u.Quantity):
 
 def _convert_to_and_validate_length_unit(unit, allow_dimensionless=False):
     """
-    raises `astropy.units.UnitsError` if not a length unit
+    raises UnitsError if not a length unit
     """
-    unit = u.Unit(unit)
+    try:
+        unit = u.Unit(unit)
+        assert (unit.is_equivalent(u.kpc) or
+                allow_dimensionless and unit == u.dimensionless_unscaled)
+    except (TypeError, AssertionError):
+        raise u.UnitsError('Unit "{0}" is not a length type'.format(unit))
 
-    if not unit.is_equivalent(u.kpc):
-        if not (allow_dimensionless and unit == u.dimensionless_unscaled):
-            raise u.UnitsError('Unit "{0}" is not a length type'.format(unit))
     return unit
 
 #<------------transformation-related utility functions----------------->
