@@ -5,13 +5,15 @@ not meant to be used directly, but instead are available as readers/writers in
 `astropy.table`. See :ref:`table_io` for more details.
 """
 
-from __future__ import print_function
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
 
 import os
 import warnings
 
 import numpy as np
 from ...utils.exceptions import AstropyUserWarning
+from ...extern import six
 
 HDF5_SIGNATURE = b'\x89HDF\r\n\x1a\n'
 
@@ -62,7 +64,7 @@ def read_table_hdf5(input, path=None):
 
     Parameters
     ----------
-    input : str or `h5py.highlevel.File` or `h5py.highlevel.Group` or `h5py.highlevel.Dataset`
+    input : str or :class:`h5py:File` or :class:`h5py:Group` or :class:`h5py:Dataset`
         If a string, the filename to read the table from. If an h5py object,
         either the file or the group object to read the table from.
     path : str
@@ -82,7 +84,7 @@ def read_table_hdf5(input, path=None):
         if path:
             try:
                 input = input[path]
-            except KeyError:
+            except (KeyError, ValueError):
                 raise IOError("Path {0} does not exist".format(path))
 
         # `input` is now either a group or a dataset. If it is a group, we
@@ -148,14 +150,20 @@ def write_table_hdf5(table, output, path=None, compression=False,
 
     Parameters
     ----------
-    output : str or `h5py.highlevel.File` or `h5py.highlevel.Group`
+    table : `~astropy.table.Table`
+        Data table that is to be written to file.
+    output : str or :class:`h5py:File` or :class:`h5py:Group`
         If a string, the filename to write the table to. If an h5py object,
         either the file or the group object to write the table to.
-    compression : bool
-        Whether to compress the table inside the HDF5 file.
     path : str
         The path to which to write the table inside the HDF5 file.
         This should be relative to the input file or group.
+    compression : bool or str or int
+        Whether to compress the table inside the HDF5 file. If set to `True`,
+        ``'gzip'`` compression is used. If a string is specified, it should be
+        one of ``'gzip'``, ``'szip'``, or ``'lzf'``. If an integer is
+        specified (in the range 0-9), ``'gzip'`` compression is used, and the
+        integer denotes the compression level.
     append : bool
         Whether to append the table to an existing HDF5 file.
     overwrite : bool
@@ -182,12 +190,12 @@ def write_table_hdf5(table, output, path=None, compression=False,
         if group:
             try:
                 output_group = output[group]
-            except KeyError:
+            except (KeyError, ValueError):
                 output_group = output.create_group(group)
         else:
             output_group = output
 
-    elif isinstance(output, basestring):
+    elif isinstance(output, six.string_types):
 
         if os.path.exists(output) and not append:
             if overwrite:
@@ -215,7 +223,12 @@ def write_table_hdf5(table, output, path=None, compression=False,
         raise IOError("Table {0} already exists".format(path))
 
     # Write the table to the file
-    dset = output_group.create_dataset(name, data=table._data, compression=compression)
+    if compression:
+        if compression is True:
+            compression = 'gzip'
+        dset = output_group.create_dataset(name, data=table._data, compression=compression)
+    else:
+        dset = output_group.create_dataset(name, data=table._data)
 
     # Write the meta-data to the file
     for key in table.meta:
