@@ -1,5 +1,7 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
+# TEST_UNICODE_LITERALS
+
 import copy
 import functools
 import sys
@@ -9,7 +11,8 @@ from datetime import datetime
 import numpy as np
 
 from ...tests.helper import pytest
-from .. import Time, ScaleValueError, erfa_time
+from .. import Time, ScaleValueError, erfa_time, TIME_SCALES
+from ...coordinates import EarthLocation
 
 
 allclose_jd = functools.partial(np.allclose, rtol=2. ** -52, atol=0)
@@ -81,10 +84,11 @@ class TestBasic():
         set isscalar as appropriate, and also subscript delta_ut1_utc, etc."""
 
         mjd = np.arange(50000, 50010)
-        t = Time(mjd, format='mjd', scale='utc')
+        t = Time(mjd, format='mjd', scale='utc', location=('45d', '50d'))
         t1 = t[3]
         assert t1.isscalar is True
         assert np.all(t1._time.jd1 == np.array([t._time.jd1[3]]))
+        assert t1.location is t.location
         t1a = Time(mjd[3], format='mjd', scale='utc')
         assert t1a.isscalar is True
         assert np.all(t1._time.jd1 == t1a._time.jd1)
@@ -94,6 +98,7 @@ class TestBasic():
         t2 = t[4:6]
         assert t2.isscalar is False
         assert np.all(t2._time.jd1 == t._time.jd1[4:6])
+        assert t2.location is t.location
         t2a = Time(t[4:6])
         assert t2a.isscalar is False
         assert np.all(t2a._time.jd1 == t._time.jd1[4:6])
@@ -106,6 +111,19 @@ class TestBasic():
         t.delta_tdb_tt = np.arange(len(t))  # Explicitly set (not testing .tdb)
         t3 = t[4:6]
         assert np.all(t3._delta_tdb_tt == t._delta_tdb_tt[4:6])
+        t4 = Time(mjd, format='mjd', scale='utc',
+                  location=(np.arange(len(mjd)), np.arange(len(mjd))))
+        t5 = t4[3]
+        assert t5.location == t4.location[3]
+        t6 = t4[4:6]
+        assert np.all(t6.location == t4.location[4:6])
+        # check it is a view
+        # (via ndarray, since quantity setter problematic for structured array)
+        allzeros = np.array((0., 0., 0.), dtype=t4.location.dtype)
+        assert t6.location.view(np.ndarray)[-1] != allzeros
+        assert t4.location.view(np.ndarray)[5] != allzeros
+        t6.location.view(np.ndarray)[-1] = allzeros
+        assert t4.location.view(np.ndarray)[5] == allzeros
 
     def test_properties(self):
         """Use properties to convert scales and formats.  Note that the UT1 to
@@ -161,15 +179,74 @@ class TestBasic():
         lat = 19.48125
         lon = -155.933222
         t = Time('2006-01-15 21:24:37.5', format='iso', scale='utc',
-                 precision=6, lat=lat, lon=lon)
+                 precision=6, location=(lon, lat))
         t.delta_ut1_utc = 0.3341  # Explicitly set one part of the xform
         assert t.utc.iso == '2006-01-15 21:24:37.500000'
         assert t.ut1.iso == '2006-01-15 21:24:37.834100'
         assert t.tai.iso == '2006-01-15 21:25:10.500000'
         assert t.tt.iso == '2006-01-15 21:25:42.684000'
         assert t.tcg.iso == '2006-01-15 21:25:43.322690'
-        assert t.tdb.iso == '2006-01-15 21:25:42.683799'
-        assert t.tcb.iso == '2006-01-15 21:25:56.893378'
+        assert t.tdb.iso == '2006-01-15 21:25:42.684373'
+        assert t.tcb.iso == '2006-01-15 21:25:56.893952'
+
+    def test_location(self):
+        """Check that location creates an EarthLocation object, and that
+        such objects can be used as arguments.
+        """
+        lat = 19.48125
+        lon = -155.933222
+        t = Time(['2006-01-15 21:24:37.5'], format='iso', scale='utc',
+                 precision=6, location=(lon, lat))
+        assert isinstance(t.location, EarthLocation)
+        location = EarthLocation(lon, lat)
+        t2 = Time(['2006-01-15 21:24:37.5'], format='iso', scale='utc',
+                  precision=6, location=location)
+        assert isinstance(t2.location, EarthLocation)
+        assert t2.location == t.location
+        t3 = Time(['2006-01-15 21:24:37.5'], format='iso', scale='utc',
+                  precision=6, location=(location.x, location.y, location.z))
+        assert isinstance(t3.location, EarthLocation)
+        assert t3.location == t.location
+
+    def test_location_array(self):
+        """Check that location arrays are checked for size and used
+        for the corresponding times.  Also checks that erfa_time.d_tdb_tt
+        can handle array-valued locations.
+        """
+
+        lat = 19.48125
+        lon = -155.933222
+        t = Time(['2006-01-15 21:24:37.5']*2, format='iso', scale='utc',
+                 precision=6, location=(lon, lat))
+        assert np.all(t.utc.iso == '2006-01-15 21:24:37.500000')
+        assert np.all(t.tdb.iso[0] == '2006-01-15 21:25:42.684373')
+        t2 = Time(['2006-01-15 21:24:37.5']*2, format='iso', scale='utc',
+                  precision=6, location=(np.array([lon, 0]),
+                                         np.array([lat, 0])))
+        assert np.all(t2.utc.iso == '2006-01-15 21:24:37.500000')
+        assert t2.tdb.iso[0] == '2006-01-15 21:25:42.684373'
+        assert t2.tdb.iso[1] != '2006-01-15 21:25:42.684373'
+        with pytest.raises(ValueError):  # 1 time, but two locations
+            Time('2006-01-15 21:24:37.5', format='iso', scale='utc',
+                 precision=6, location=(np.array([lon, 0]),
+                                        np.array([lat, 0])))
+        with pytest.raises(ValueError):  # 3 times, but two locations
+            Time(['2006-01-15 21:24:37.5']*3, format='iso', scale='utc',
+                 precision=6, location=(np.array([lon, 0]),
+                                        np.array([lat, 0])))
+
+    def test_all_transforms(self):
+        """Test that all transforms work.  Does not test correctness,
+        except reversibility [#2074]"""
+        lat = 19.48125
+        lon = -155.933222
+        for scale1 in TIME_SCALES:
+            t1 = Time('2006-01-15 21:24:37.5', format='iso', scale=scale1,
+                      lat=lat, lon=lon)
+            for scale2 in TIME_SCALES:
+                t2 = getattr(t1, scale2)
+                t21 = getattr(t2, scale1)
+                assert allclose_jd(t21.jd, t1.jd)
 
     def test_creating_all_formats(self):
         """Create a time object using each defined format"""
@@ -181,10 +258,13 @@ class TestBasic():
         Time('B1950.0', format='byear_str', scale='tai')
         Time('J2000.0', format='jyear_str', scale='tai')
         Time('2000-01-01 12:23:34.0', format='iso', scale='tai')
+        Time('2000-01-01 12:23:34.0Z', format='iso', scale='utc')
         Time('2000-01-01T12:23:34.0', format='isot', scale='tai')
+        Time('2000-01-01T12:23:34.0Z', format='isot', scale='utc')
         Time(2400000.5, 51544.0333981, format='jd', scale='tai')
         Time(0.0, 51544.0333981, format='mjd', scale='tai')
         Time('2000:001:12:23:34.0', format='yday', scale='tai')
+        Time('2000:001:12:23:34.0Z', format='yday', scale='utc')
         dt = datetime(2000, 1, 2, 3, 4, 5, 123456)
         Time(dt, format='datetime', scale='tai')
         Time([dt, dt], format='datetime', scale='tai')
@@ -237,6 +317,8 @@ class TestBasic():
             Time([50000.0], ['bad'], format='mjd', scale='tai')
         with pytest.raises(ValueError):
             Time(50000.0, 'bad', format='mjd', scale='tai')
+        with pytest.raises(ValueError):
+            Time('2005-08-04T00:01:02.000Z', scale='tai')
 
     def test_utc_leap_sec(self):
         """Time behaves properly near or in UTC leap second.  This
@@ -388,17 +470,30 @@ class TestSubFormat():
         assert t.scale == 'utc'
 
         # Check that bad scale is caught when format is specified
-        with pytest.raises(ScaleValueError):
-            Time(1950.0, format='byear')
         with pytest.raises(ScaleValueError):
             Time(1950.0, format='byear', scale='bad scale')
 
         # Check that bad scale is caught when format is auto-determined
-        with pytest.raises(ScaleValueError):
-            Time('2000:001:00:00:00')
         with pytest.raises(ScaleValueError):
             Time('2000:001:00:00:00', scale='bad scale')
 
+    def test_scale_default(self):
+        """Test behavior when no scale is provided"""
+        # These first three are TimeFromEpoch and have an intrinsic time scale
+        t = Time(100.0, format='cxcsec')
+        assert t.scale == 'tt'
+        t = Time(100.0, format='unix')
+        assert t.scale == 'utc'
+        t = Time(100.0, format='gps')
+        assert t.scale == 'tai'
+
+        for date in ('J2000', '2000:001', '2000-01-01T00:00:00'):
+            t = Time(date)
+            assert t.scale == 'utc'
+
+        t = Time(2000.1, format='byear')
+        assert t.scale == 'utc'
+
     def test_epoch_times(self):
         """Test time formats derived from EpochFromTime"""
         t = Time(0.0, format='cxcsec', scale='tai')
@@ -417,8 +512,8 @@ class TestSubFormat():
 
         # Value from:
         #   d = datetime.datetime(2000, 1, 1)
-        #   matplotlib.dates.date2num(d)
-        t = Time('2000-01-01 00:00:00', scale='tai')
+        #   matplotlib.pylab.dates.date2num(d)
+        t = Time('2000-01-01 00:00:00', scale='utc')
         assert np.allclose(t.plot_date, 730120.0, atol=1e-5, rtol=0)
 
         # Round trip through epoch time
@@ -478,29 +573,44 @@ class TestCopyReplicate():
 
     def test_replicate(self):
         """Test replicate method"""
-        t = Time('2000:001', format='yday', scale='tai')
+        t = Time('2000:001', format='yday', scale='tai',
+                 location=('45d', '45d'))
         t_yday = t.yday
+        t_loc_x = t.location.x.copy()
         t2 = t.replicate()
         assert t.yday == t2.yday
         assert t.format == t2.format
         assert t.scale == t2.scale
-        # This is not allowed publicly, but here we hack the internal time values
-        # to show that t and t2 are sharing references.
+        assert t.location == t2.location
+        # This is not allowed publicly, but here we hack the internal time
+        # and location values to show that t and t2 are sharing references.
         t2._time.jd1 += 100.0
         assert t.yday == t2.yday
         assert t.yday != t_yday  # prove that it changed
+        t2_loc_x_view = t2.location.x
+        t2_loc_x_view[()] = 0  # use 0 to avoid having to give units
+        assert t2.location.x == t2_loc_x_view
+        assert t.location.x == t2.location.x
+        assert t.location.x != t_loc_x  # prove that it changed
 
     def test_copy(self):
         """Test copy method"""
-        t = Time('2000:001', format='yday', scale='tai')
+        t = Time('2000:001', format='yday', scale='tai',
+                 location=('45d', '45d'))
         t_yday = t.yday
+        t_loc_x = t.location.x.copy()
         t2 = t.copy()
         assert t.yday == t2.yday
-        # This is not allowed publicly, but here we hack the internal time values
-        # to show that t and t2 are not sharing references.
+        # This is not allowed publicly, but here we hack the internal time
+        # and location values to show that t and t2 are not sharing references.
         t2._time.jd1 += 100.0
         assert t.yday != t2.yday
         assert t.yday == t_yday  # prove that it did not change
+        t2_loc_x_view = t2.location.x
+        t2_loc_x_view[()] = 0  # use 0 to avoid having to give units
+        assert t2.location.x == t2_loc_x_view
+        assert t.location.x != t2.location.x
+        assert t.location.x == t_loc_x  # prove that it changed
 
 
 def test_python_builtin_copy():
@@ -545,3 +655,16 @@ def test_now():
 def test_dir():
     t = Time('2000:001', format='yday', scale='tai')
     assert 'utc' in dir(t)
+
+
+def test_TimeFormat_scale():
+    """guard against recurrence of #1122, where TimeFormat class looses uses
+    attributes (delta_ut1_utc here), preventing conversion to unix, cxc"""
+    t = Time('1900-01-01', scale='ut1')
+    t.delta_ut1_utc = 0.0
+    t.unix
+    assert t.unix == t.utc.unix
+
+def test_scale_conversion():
+    with pytest.raises(ScaleValueError):
+        t = Time(Time.now().cxcsec, format='cxcsec', scale='ut1')
