@@ -36,7 +36,8 @@ def_unit(['micron'], um, namespace=_ns,
 
 def_unit(['Angstrom', 'AA', 'angstrom'], 0.1 * nm, namespace=_ns,
          doc="ångström: 10 ** -10 m",
-         format={'latex': r'\AA', 'unicode': 'Å', 'vounit': 'angstrom'})
+         format={'latex': r'\overset{\circ}{A}', 'unicode': 'Å',
+                 'vounit': 'angstrom'})
 
 
 ###########################################################################
@@ -130,6 +131,9 @@ def_unit(['mol', 'mole'], namespace=_ns, prefixes=True,
 def_unit(
     ['K', 'Kelvin'], namespace=_ns, prefixes=True,
     doc="Kelvin: temperature with a null point at absolute zero.")
+def_unit(
+    ['deg_C', 'Celsius'], namespace=_ns, doc='Degrees Celsius',
+    format={'latex': r'{}^{\circ}C', 'unicode': '°C'})
 
 
 ###########################################################################
@@ -148,7 +152,7 @@ def_unit(['eV', 'electronvolt'], _si.e.value * J, namespace=_ns, prefixes=True,
          doc="Electron Volt")
 def_unit(['Pa', 'Pascal', 'pascal'], J * m ** -3, namespace=_ns, prefixes=True,
          doc="Pascal: pressure")
-def_unit(['bar'], 100 * Pa, namespace=_ns,
+def_unit(['bar'], 1e5 * Pa, namespace=_ns,
          doc="bar: pressure")
 
 
@@ -168,7 +172,7 @@ def_unit(['C', 'coulomb'], A * s, namespace=_ns, prefixes=True,
          doc="coulomb: electric charge")
 def_unit(['V', 'Volt', 'volt'], J * C ** -1, namespace=_ns, prefixes=True,
          doc="Volt: electric potential or electromotive force")
-def_unit(['Ohm', 'ohm'], V * A ** -1, namespace=_ns, prefixes=True,
+def_unit((['Ohm', 'ohm'], ['Ohm']), V * A ** -1, namespace=_ns, prefixes=True,
          doc="Ohm: electrical resistance",
          format={'latex': r'\Omega', 'unicode': 'Ω'})
 def_unit(['S', 'Siemens', 'siemens'], A * V ** -1, namespace=_ns,
