@@ -306,7 +306,6 @@ class TestQuantityMathFuncs(object):
         assert np.all(q == np.array([-1., -2., -3.]))
         assert not isinstance(q, u.Quantity)
 
-
     def test_ldexp_scalar(self):
         assert np.ldexp(4. * u.m, 2) == 16. * u.m
 
@@ -315,12 +314,11 @@ class TestQuantityMathFuncs(object):
                       == np.array([8., 8., 6.]) * u.m)
 
     def test_ldexp_invalid(self):
-        with pytest.raises(TypeError) as exc:
+        with pytest.raises(TypeError):
             np.ldexp(3. * u.m, 4.)
-        # built-in TypeError, so can't check content of exception
-        with pytest.raises(TypeError) as exc:
-            np.ldexp(3., 4 * u.m)
-        assert "Cannot use ldexp" in exc.value.args[0]
+
+        with pytest.raises(TypeError):
+            np.ldexp(3., u.Quantity(4, u.m, dtype=int))
 
     @pytest.mark.parametrize('function', (np.exp, np.expm1, np.exp2,
                                           np.log, np.log2, np.log10, np.log1p))
@@ -519,6 +517,29 @@ class TestInplaceUfuncs(object):
         assert check2.unit == u.dimensionless_unscaled
         assert_allclose(s.value, s2.value)
 
+    @pytest.mark.skipif("NUMPY_LT_1P6")
+    @pytest.mark.parametrize(('value'), [1., np.arange(10.)])
+    def test_one_argument_ufunc_inplace_2(self, value):
+        """Check inplace works with non-quantity input and quantity output"""
+        s = value * u.m
+        check = s
+        np.absolute(value, out=s)
+        assert check is s
+        assert np.all(check.value == np.absolute(value))
+        assert check.unit is u.dimensionless_unscaled
+        np.sqrt(value, out=s)
+        assert check is s
+        assert np.all(check.value == np.sqrt(value))
+        assert check.unit is u.dimensionless_unscaled
+        np.exp(value, out=s)
+        assert check is s
+        assert np.all(check.value == np.exp(value))
+        assert check.unit is u.dimensionless_unscaled
+        np.arcsin(value/10., out=s)
+        assert check is s
+        assert np.all(check.value == np.arcsin(value/10.))
+        assert check.unit is u.radian
+
     @pytest.mark.parametrize(('value'), [1., np.arange(10.)])
     def test_one_argument_two_output_ufunc_inplace(self, value):
         v = 100. * value * u.cm / u.m
@@ -582,6 +603,18 @@ class TestInplaceUfuncs(object):
     @pytest.mark.skipif("NUMPY_LT_1P6")
     def test_two_argument_ufunc_inplace_3(self):
         s = np.array([1., 2., 3.]) * u.dimensionless_unscaled
+        np.add(np.array([1., 2., 3.]), np.array([1., 2., 3.]) * 2., out=s)
+        assert np.all(s.value == np.array([3., 6., 9.]))
+        assert s.unit is u.dimensionless_unscaled
         np.arctan2(np.array([1., 2., 3.]), np.array([1., 2., 3.]) * 2., out=s)
         assert_allclose(s.value, np.arctan2(1., 2.))
-        assert s.unit == u.radian
+        assert s.unit is u.radian
+
+    def test_ufunc_inplace_non_contiguous_data(self):
+        # ensure inplace works also for non-contiguous data (closes #1834)
+        s = np.arange(10.) * u.m
+        s_copy = s.copy()
+        s2 = s[::2]
+        s2 += 1. * u.cm
+        assert np.all(s[::2] > s_copy[::2])
+        assert np.all(s[1::2] == s_copy[1::2])
