@@ -161,6 +161,25 @@ b_order = """
 ``int`` (read-only) Order of the polynomial (``B_ORDER``).
 """
 
+bounds_check = """
+bounds_check(pix2world, world2pix)
+
+Parameters
+----------
+pix2world : bool, optional
+    When `True`, enable bounds checking for the pixel-to-world (p2x)
+    transformations.  Default is `True`.
+
+world2pix : bool, optional
+    When `True`, enable bounds checking for the world-to-pixel (s2x)
+    transformations.  Default is `True`.
+
+Notes
+-----
+Note that by default (without calling `bounds_check`) strict bounds
+checking is enabled.
+"""
+
 bp = """
 ``double array[bp_order+1][bp_order+1]`` Focal plane to pixel
 transformation matrix.
@@ -265,6 +284,43 @@ relevant column number.
 It should be set to zero for an image header or pixel list.
 """
 
+compare = """
+compare(other, cmp=0)
+
+Compare two Wcsprm objects for equality.
+
+Parameters
+----------
+
+other : Wcsprm
+    The other Wcsprm object to compare to.
+
+cmp : int, optional
+    A bit field controlling the strictness of the comparison.  When 0,
+    (the default), all fields must be identical.
+
+    The following constants may be or'ed together to loosen the
+    comparison.
+
+    - ``WCSCOMPARE_ANCILLARY``: Ignores ancillary keywords that don't
+      change the WCS transformation, such as ``DATE-OBS`` or
+      ``EQUINOX``.
+
+    - ``WCSCOMPARE_TILING``: Ignore integral differences in
+      ``CRPIXja``.  This is the 'tiling' condition, where two WCSes
+      cover different regions of the same map projection and align on
+      the same map grid.
+
+    - ``WCSCOMPARE_CRPIX``: Ignore any differences at all in
+      ``CRPIXja``.  The two WCSes cover different regions of the same
+      map projection but may not align on the same grid map.
+      Overrides ``WCSCOMPARE_TILING``.
+
+Returns
+-------
+equal : bool
+"""
+
 convert = """
 convert(array)
 
@@ -279,7 +335,7 @@ Has the dimensions::
 
     (K_M, ... K_2, K_1, M)
 
-(see `~astropy.wcs._astropy.wcs.Tabprm.K`) i.e. with the `M` dimension
+(see `~astropy.wcs.Tabprm.K`) i.e. with the `M` dimension
 varying fastest so that the `M` elements of a coordinate vector are
 stored contiguously in memory.
 """
@@ -412,13 +468,13 @@ commonly used non-standard units specifications but this must be done
 as a separate step before invoking `~astropy.wcs.Wcsprm.set`.
 
 For celestial axes, if `~astropy.wcs.Wcsprm.cunit` is not blank,
-`~astropy.wcs.Wcsprm.set` uses `wcsunits` to parse it and scale
+`~astropy.wcs.Wcsprm.set` uses ``wcsunits`` to parse it and scale
 `~astropy.wcs.Wcsprm.cdelt`, `~astropy.wcs.Wcsprm.crval`, and
 `~astropy.wcs.Wcsprm.cd` to decimal degrees.  It then resets
 `~astropy.wcs.Wcsprm.cunit` to ``"deg"``.
 
 For spectral axes, if `~astropy.wcs.Wcsprm.cunit` is not blank,
-`~astropy.wcs.Wcsprm.set` uses `wcsunits` to parse it and scale
+`~astropy.wcs.Wcsprm.set` uses ``wcsunits`` to parse it and scale
 `~astropy.wcs.Wcsprm.cdelt`, `~astropy.wcs.Wcsprm.crval`, and
 `~astropy.wcs.Wcsprm.cd` to SI units.  It then resets
 `~astropy.wcs.Wcsprm.cunit` accordingly.
@@ -480,7 +536,7 @@ already set.
 Alternatively, if `~astropy.wcs.Wcsprm.mjdobs` is set and
 `~astropy.wcs.Wcsprm.dateobs` isn't, then `~astropy.wcs.Wcsprm.datfix`
 derives `~astropy.wcs.Wcsprm.dateobs` from it.  If both are set but
-disagree by more than half a day then `ValueError` is raised.
+disagree by more than half a day then `~.exceptions.ValueError` is raised.
 
 Returns
 -------
@@ -494,7 +550,7 @@ array.
 
 Array of interpolated indices into the coordinate array such that
 Upsilon_m, as defined in Paper III, is equal to
-(`~astropy.wcs._astropy.wcs.Tabprm.p0` [m] + 1) + delta[m].
+(`~astropy.wcs.Tabprm.p0` [m] + 1) + delta[m].
 """
 
 det2im = """
@@ -515,7 +571,7 @@ dims = """
 ``int array[ndim]`` (read-only)
 
 The dimensions of the tabular array
-`~astropy.wcs._astropy.wcs.Wtbarr.data`.
+`~astropy.wcs.Wtbarr.data`.
 """
 
 DistortionLookupTable = """
@@ -570,7 +626,7 @@ dimensions::
 
     (K_M, ... K_2, 2, M)
 
-(see `~astropy.wcs._astropy.wcs.Tabprm.K`).  The minimum is recorded
+(see `~astropy.wcs.Tabprm.K`).  The minimum is recorded
 in the first element of the compressed K_1 dimension, then the
 maximum.  This array is used by the inverse table lookup function to
 speed up table searches.
@@ -619,7 +675,7 @@ keysel : sequence of flags
 
 Returns
 -------
-wcs_list : list of `~astropy.wcs._astropy.wcs.Wcsprm` objects
+wcs_list : list of `~astropy.wcs.Wcsprm` objects
 """
 
 fix = """
@@ -633,8 +689,9 @@ Applies all of the corrections handled separately by
 Parameters
 ----------
 
-translate_units : str
-    Do potentially unsafe translations of non-standard unit strings.
+translate_units : str, optional
+    Specify which potentially unsafe translations of non-standard unit
+    strings to perform.  By default, performs all.
 
     Although ``"S"`` is commonly used to represent seconds, its
     translation to ``"s"`` is potentially unsafe since the standard
@@ -654,7 +711,7 @@ translate_units : str
     Thus ``''`` doesn't do any unsafe translations, whereas ``'shd'``
     does all of them.
 
-naxis : int array[naxis]
+naxis : int array[naxis], optional
     Image axis lengths.  If this array is set to zero or ``None``,
     then `~astropy.wcs.Wcsprm.cylfix` will not be invoked.
 
@@ -845,13 +902,6 @@ Alias for `~astropy.wcs.Wcsprm.has_pc`.  Maintained for backward
 compatibility.
 """
 
-have = """
-``string`` The name of the unit being converted from.
-
-This value always uses standard unit names, even if the
-`UnitConverter` was initialized with a non-standard unit name.
-"""
-
 i = """
 ``int`` (read-only)
 
@@ -939,7 +989,7 @@ Array axis number for index vectors.
 map = """
 ``int array[M]`` Association between axes.
 
-A vector of length `~astropy.wcs._astropy.wcs.Tabprm.M` that defines
+A vector of length `~astropy.wcs.Tabprm.M` that defines
 the association between axis *m* in the *M*-dimensional coordinate
 array (1 <= *m* <= *M*) and the indices of the intermediate world
 coordinate and world coordinate arrays.
@@ -1172,14 +1222,10 @@ reference frame.
 An undefined value is represented by NaN.
 """
 
-offset = """
-``double`` The offset of the unit conversion.
-"""
-
 p0 = """
 ``int array[M]`` Interpolated indices into the coordinate array.
 
-Vector of length `~astropy.wcs._astropy.wcs.Tabprm.M` of interpolated
+Vector of length `~astropy.wcs.Tabprm.M` of interpolated
 indices into the coordinate array such that Upsilon_m, as defined in
 Paper III, is equal to ``(p0[m] + 1) + delta[m]``.
 """
@@ -1360,10 +1406,6 @@ piximg_matrix = """
 the ``CDELTia`` diagonal matrix and the ``PCi_ja`` matrix.
 """
 
-power = """
-``double`` The exponent of the unit conversion.
-"""
-
 print_contents = """
 print_contents()
 
@@ -1377,7 +1419,7 @@ To get a string of the contents, use `repr`.
 print_contents_tabprm = """
 print_contents()
 
-Print the contents of the `~astropy.wcs._astropy.wcs.Tabprm` object to
+Print the contents of the `~astropy.wcs.Tabprm` object to
 stdout.  Probably only useful for debugging purposes, and may be
 removed in the future.
 
@@ -1477,14 +1519,10 @@ astropy.wcs.Wcsprm.lat, astropy.wcs.Wcsprm.lng
     Definition of the latitude and longitude axes
 """.format(__.ORIGIN())
 
-scale = """
-``double`` The scaling factor for the unit conversion.
-"""
-
 sense = """
 ``int array[M]`` +1 if monotonically increasing, -1 if decreasing.
 
-A vector of length `~astropy.wcs._astropy.wcs.Tabprm.M` whose elements
+A vector of length `~astropy.wcs.Tabprm.M` whose elements
 indicate whether the corresponding indexing vector is monotonically
 increasing (+1), or decreasing (-1).
 """
@@ -1553,9 +1591,9 @@ InvalidTabularParameters
 """
 
 set_ps = """
-set_ps(list)
+set_ps(ps)
 
-Sets `PSi_ma` keywords for each *i* and *m*.
+Sets ``PSi_ma`` keywords for each *i* and *m*.
 
 Parameters
 ----------
@@ -1576,9 +1614,9 @@ astropy.wcs.Wcsprm.get_ps
 """
 
 set_pv = """
-set_pv(list)
+set_pv(pv)
 
-Sets `PVi_ma` keywords for each *i* and *m*.
+Sets ``PVi_ma`` keywords for each *i* and *m*.
 
 Parameters
 ----------
@@ -1874,13 +1912,13 @@ for example::
       -(WCSSUB_SPECTRAL | WCSSUB_STOKES)])
 
 The last of these specifies all axis types other than spectral or
-Stokes.  Extraction is done in the order specified by `axes`, i.e. a
+Stokes.  Extraction is done in the order specified by ``axes``, i.e. a
 longitude axis (if present) would be extracted first (via ``axes[0]``)
 and not subsequently (via ``axes[3]``).  Likewise for the latitude and
 cubeface axes in this example.
 
 The number of dimensions in the returned object may be less than or
-greater than the length of `axes`.  However, it will never exceed the
+greater than the length of ``axes``.  However, it will never exceed the
 number of axes in the input image.
 """
 
@@ -1935,7 +1973,7 @@ number of respects:
 
     2. Deprecated (e.g. ``CROTAn``) or non-standard usage will be
        translated to standard (this is partially dependent on whether
-       `fix` was applied).
+       ``fix`` was applied).
 
     3. Quantities will be converted to the units used internally,
        basically SI with the addition of degrees.
@@ -1958,7 +1996,7 @@ number of respects:
 
 Keywords can be translated between the image array, binary table, and
 pixel lists forms by manipulating the `~astropy.wcs.Wcsprm.colnum` or
-`~astropy.wcs.Wcsprm.colax` members of the `~astropy.wcs.Wcsprm.WCS`
+`~astropy.wcs.Wcsprm.colax` members of the `~astropy.wcs.WCS`
 object.
 
 Parameters
@@ -1989,97 +2027,6 @@ ttype = """
 the wcstab array.
 """
 
-UnitConverter = """
-UnitConverter(have, want, translate_units='')
-
-An object for converting from one system of units to another.
-
-Use the returned object's `~astropy.wcs.UnitConverter.convert` method
-to convert values from *have* to *want*.
-
-This function is permissive in accepting whitespace in all contexts in
-a units specification where it does not create ambiguity (e.g. not
-between a metric prefix and a basic unit string), including in strings
-like ``"log (m ** 2)"`` which is formally disallowed.
-
-.. note:: Deprecated in Astropy 0.2
-
-   `UnitConverter` will be removed in a future version of astropy.
-   The `astropy.units` package should be used instead.
-
-Parameters
-----------
-
-have : str
-    FITS unit string to convert from, with or without surrounding
-    square brackets (for inline specifications); text following the
-    closing bracket is ignored.
-
-want : str
-    FITS unit string to convert to, with or without surrounding square
-    brackets (for inline specifications); text following the closing
-    bracket is ignored.
-
-ctrl : str, optional
-    Do potentially unsafe translations of non-standard unit strings.
-
-    Although ``\"S\"`` is commonly used to represent seconds, its
-    recognizes ``\"S\"`` formally as Siemens, however rarely that may
-    be translation to ``\"s\"`` is potentially unsafe since the
-    standard used.  The same applies to ``\"H\"`` for hours (Henry),
-    and ``\"D\"`` for days (Debye).
-
-    This string controls what to do in such cases, and is
-    case-insensitive.
-
-    - If the string contains ``"s"``, translate ``"S"`` to ``"s"``.
-
-    - If the string contains ``"h"``, translate ``"H"`` to ``"h"``.
-
-    - If the string contains ``"d"``, translate ``"D"`` to ``"d"``.
-
-    Thus ``''`` doesn't do any unsafe translations, whereas ``'shd'``
-    does all of them.
-
-Raises
-------
-ValueError
-    Invalid numeric multiplier.
-
-SyntaxError
-    Dangling binary operator.
-
-SyntaxError
-    Invalid symbol in INITIAL context.
-
-SyntaxError
-    Function in invalid context.
-
-SyntaxError
-    Invalid symbol in EXPON context.
-
-SyntaxError
-    Unbalanced bracket.
-
-SyntaxError
-    Unbalanced parenthesis.
-
-SyntaxError
-    Consecutive binary operators.
-
-SyntaxError
-    Internal parser error.
-
-SyntaxError
-    Non-conformant unit specifications.
-
-SyntaxError
-    Non-conformant functions.
-
-ValueError
-    Potentially unsafe translation.
-"""
-
 unitfix = """
 unitfix(translate_units='')
 
@@ -2140,13 +2087,6 @@ See also
 astropy.wcs.Wcsprm.specsys, astropy.wcs.Wcsprm.ssysobs
 """
 
-want = """
-``string`` The name of the unit being converted to.
-
-This value always uses standard unit names, even if the
-`UnitConverter` was initialized with a non-standard unit name.
-"""
-
 wcs = """
 A `~astropy.wcs.Wcsprm` object to perform the basic `wcslib`_ WCS
 transformation.
@@ -2159,7 +2099,7 @@ Wcs objects amalgamate basic WCS (as provided by `wcslib`_), with
 `SIP`_ and `Paper IV`_ distortion operations.
 
 To perform all distortion corrections and WCS tranformation, use
-`all_pix2world`.
+``all_pix2world``.
 
 Parameters
 ----------
@@ -2180,6 +2120,14 @@ Wcsprm(header=None, key=' ', relax=False, naxis=2, keysel=0, colsel=None)
 `~astropy.wcs.Wcsprm` is a direct wrapper around `wcslib`_.  It
 provides access to the core WCS transformations that it supports.
 
+.. note::
+    The members of this object correspond roughly to the key/value
+    pairs in the FITS header.  However, they are adjusted and
+    normalized in a number of ways that make performing the WCS
+    transformation easier.  Therefore, they can not be relied upon to
+    get the original values in the header.  For that, use
+    `astropy.io.fits.Header` directly.
+
 The FITS header parsing enforces correct FITS "keyword = value" syntax
 with regard to the equals sign occurring in columns 9 and 10.
 However, it does recognize free-format character (NOST 100-2.0,
