@@ -966,16 +966,12 @@ struct module_state {
     INITERROR;
 
   import_array();
-#if defined(_MSC_VER)
-  fill_docstrings();
-#endif
 
   if (_setup_api(m)                 ||
       _setup_str_list_proxy_type(m) ||
       _setup_unit_list_proxy_type(m)||
       _setup_wcsprm_type(m)         ||
       _setup_tabprm_type(m)         ||
-      _setup_units_type(m)          ||
       /* _setup_wtbarr_type(m)         || */
       _setup_distortion_type(m)     ||
       _setup_sip_type(m)            ||
